@@ -1,6 +1,7 @@
 /* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  *  Copyright (C) 2005 Takuro Ashie
+ *  Copyright (C) 2006 Juernjakob Harder
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +24,6 @@
 
 #include <stdlib.h>
 #include <math.h>
-#include <tomoe/tomoe.h>
 
 #include "tomoe_canvas.h"
 
@@ -50,10 +50,13 @@ struct _TomoeCanvasPriv
     GdkPixmap       *pixmap;
     GList           *stroke;
     GList           *stroke_list;
-    candidate      **candidates;
+    tomoe_array*     candidates;
     unsigned int     candidates_len;
     gint             auto_find_time;
     guint            auto_find_id;
+    tomoe_db        *database;
+    tomoe_glyph     *preview;
+    gboolean         locked;
 };
 #define TOMOE_CANVAS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TOMOE_TYPE_CANVAS, TomoeCanvasPriv))
 
@@ -77,9 +80,10 @@ static void   tomoe_canvas_append_point         (TomoeCanvas       *canvas,
                                                  gint               x,
                                                  gint               y);
 static void   tomoe_canvas_draw_line            (TomoeCanvas       *canvas,
-                                                 point             *p1,
-                                                 point             *p2,
-                                                 gboolean           draw);
+                                                 tomoe_point       *p1,
+                                                 tomoe_point       *p2,
+                                                 gboolean           draw,
+                                                 GdkColor          *color);
 static void   tomoe_canvas_draw_background      (TomoeCanvas       *canvas,
                                                  gboolean           draw);
 static void   tomoe_canvas_draw_axis            (TomoeCanvas       *canvas);
@@ -89,6 +93,8 @@ static void   tomoe_canvas_resize               (TomoeCanvas       *canvas,
 static void   tomoe_canvas_position             (TomoeCanvas       *canvas,
                                                  gint               dx,
                                                  gint               dy);
+
+static void   _init_gc                          (TomoeCanvas       *canvas);
 static gint   get_distance                      (GList             *first_node,
                                                  GList             *last_node,
                                                  GList            **most_node);
@@ -97,13 +103,24 @@ static GList *get_vertex                        (GList             *first_node,
 static void   draw_stroke                       (GList             *stroke,
                                                  TomoeCanvas       *canvas,
                                                  guint             *index);
+static void   draw_preview_stroke               (TomoeCanvas       *canvas,
+                                                 tomoe_stroke      *stroke,
+                                                 guint             *index);
 static void   draw_annotate                     (GList             *stroke,
                                                  TomoeCanvas       *canvas,
-                                                 guint              index);
+                                                 guint              index,
+                                                 GdkColor          *color);
+static void   draw_preview_annotate             (TomoeCanvas       *canvas,
+                                                 tomoe_stroke      *stroke,
+                                                 guint              inde,
+                                                 GdkColor          *color);
+static void   on_size_allocate                  (GtkWidget         *widget,
+                                                 GtkAllocation     *allocation,
+                                                 gpointer           user_data);
 
-static GList *instance_list = NULL;
-static guint canvas_signals[LAST_SIGNAL] = { 0 };
-static GtkDrawingAreaClass *parent_class = NULL;
+static GList*               instance_list = NULL;
+static guint                canvas_signals[LAST_SIGNAL] = { 0 };
+static GtkDrawingAreaClass* parent_class = NULL;
 
 GType
 tomoe_canvas_get_type (void)
@@ -202,6 +219,7 @@ tomoe_canvas_init (TomoeCanvas *canvas)
     GtkWidget *widget = GTK_WIDGET (canvas);
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     PangoFontDescription *font_desc;
+    gint width, height;
 
     gtk_widget_set_events (widget,
                            GDK_EXPOSURE_MASK |
@@ -214,6 +232,10 @@ tomoe_canvas_init (TomoeCanvas *canvas)
     font_desc = pango_font_description_from_string ("Sans 12");
     gtk_widget_modify_font (widget, font_desc);
 
+    g_signal_connect (G_OBJECT (widget), "size-allocate",
+                      G_CALLBACK (on_size_allocate),
+                      (gpointer) NULL);
+
     priv->size              = TOMOE_CANVAS_DEFAULT_SIZE;
     priv->handwrite_line_gc = NULL;
     priv->adjust_line_gc    = NULL;
@@ -226,14 +248,18 @@ tomoe_canvas_init (TomoeCanvas *canvas)
     priv->candidates_len    = 0;
     priv->auto_find_time    = 0;
     priv->auto_find_id      = 0;
+    priv->preview           = NULL;
+    priv->locked            = FALSE;
 
-    /* FIXME: customize size */
-    tomoe_canvas_set_size (canvas, 300);
+    gtk_widget_get_size_request(GTK_WIDGET (canvas), &width, &height);
+    if (width == -1)
+        priv->size = TOMOE_CANVAS_DEFAULT_SIZE;
+    else
+        priv->size = width;
+/*    tomoe_canvas_set_size (canvas, 300);*/
 
-    gtk_drawing_area_size (GTK_DRAWING_AREA (canvas), priv->size, priv->size);
+    gtk_widget_set_size_request (GTK_WIDGET (canvas), priv->size, priv->size);
 
-    if (!instance_list)
-        tomoe_init ();
     instance_list = g_list_append (instance_list, (gpointer) canvas);
 }
 
@@ -263,30 +289,28 @@ tomoe_canvas_free_stroke_list (TomoeCanvas *canvas)
 static void
 tomoe_canvas_dispose (GObject *object)
 {
-    TomoeCanvas *canvas = TOMOE_CANVAS (canvas);
+    TomoeCanvas *canvas = TOMOE_CANVAS (object);
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
     instance_list = g_list_remove (instance_list, (gpointer) canvas);
-    if (!instance_list)
-        tomoe_term ();
 
     if (priv->handwrite_line_gc) {
-        gdk_gc_unref (priv->handwrite_line_gc);
+        g_object_unref (priv->handwrite_line_gc);
         priv->handwrite_line_gc = NULL;
     }
 
     if (priv->adjust_line_gc) {
-        gdk_gc_unref (priv->adjust_line_gc);
+        g_object_unref (priv->adjust_line_gc);
         priv->adjust_line_gc = NULL;
     }
 
     if (priv->annotate_gc) {
-        gdk_gc_unref (priv->annotate_gc);
+        g_object_unref (priv->annotate_gc);
         priv->annotate_gc = NULL;
     }
 
     if (priv->axis_gc) {
-        gdk_gc_unref (priv->axis_gc);
+        g_object_unref (priv->axis_gc);
         priv->axis_gc = NULL;
     }
 
@@ -296,13 +320,13 @@ tomoe_canvas_dispose (GObject *object)
     }
 
     if (priv->candidates) {
-        tomoe_free_matched (priv->candidates, priv->candidates_len);
+        tomoe_array_free (priv->candidates);
         priv->candidates     = NULL;
         priv->candidates_len = 0;
     }
 
     if (priv->auto_find_id) {
-        gtk_timeout_remove (priv->auto_find_id);
+        g_source_remove (priv->auto_find_id);
         priv->auto_find_id = 0;
     }
 
@@ -353,7 +377,6 @@ tomoe_canvas_expose_event (GtkWidget *widget, GdkEventExpose *event)
     if (GTK_WIDGET_CLASS(parent_class)->expose_event)
         retval = GTK_WIDGET_CLASS(parent_class)->expose_event (widget,
                                                                event);
-
     gdk_draw_drawable(widget->window,
                       widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                       priv->pixmap,
@@ -371,8 +394,10 @@ tomoe_canvas_button_press_event (GtkWidget *widget, GdkEventButton *event)
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     gboolean retval = FALSE;
 
+    if (priv->locked) return retval;
+
     if (priv->auto_find_id) {
-        gtk_timeout_remove (priv->auto_find_id);
+        g_source_remove (priv->auto_find_id);
         priv->auto_find_id = 0;
     }
 
@@ -398,6 +423,12 @@ tomoe_canvas_button_release_event (GtkWidget *widget, GdkEventButton *event)
     TomoeCanvas *canvas = TOMOE_CANVAS (widget);
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     gboolean retval = FALSE;
+    GdkColor color;
+    color.red   = 0x8000;
+    color.green = 0x0000;
+    color.blue  = 0x0000;
+
+    if (priv->locked) return retval;
 
     if (event->button != 1)
         return retval;
@@ -405,7 +436,7 @@ tomoe_canvas_button_release_event (GtkWidget *widget, GdkEventButton *event)
     if (!priv->stroke)
         return retval;
 
-    draw_annotate (priv->stroke, canvas, g_list_length (priv->stroke_list) + 1);
+    draw_annotate (priv->stroke, canvas, g_list_length (priv->stroke_list) + 1, &color);
 
     priv->stroke_list = g_list_append(priv->stroke_list,
                                       priv->stroke);
@@ -414,13 +445,13 @@ tomoe_canvas_button_release_event (GtkWidget *widget, GdkEventButton *event)
     g_signal_emit (G_OBJECT (widget), canvas_signals[STROKE_ADDED_SIGNAL], 0);
 
     if (priv->auto_find_id) {
-        gtk_timeout_remove (priv->auto_find_id);
+        g_source_remove (priv->auto_find_id);
         priv->auto_find_id = 0;
     }
     if (priv->auto_find_time > 0) {
-        priv->auto_find_id = gtk_timeout_add (priv->auto_find_time,
-                                              timeout_auto_find,
-                                              (gpointer)canvas);
+        priv->auto_find_id = g_timeout_add (priv->auto_find_time,
+                                            timeout_auto_find,
+                                            (gpointer)canvas);
     } else if (priv->auto_find_time == 0) {
         tomoe_canvas_find (canvas);
     }
@@ -437,6 +468,8 @@ tomoe_canvas_motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
     GdkModifierType state;
     gboolean retval = FALSE;
 
+    if (priv->locked) return retval;
+
     if (!priv->stroke)
         return retval;
 
@@ -460,17 +493,21 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
 {
     TomoeCanvasPriv *priv;
     GtkWidget *widget = GTK_WIDGET (canvas);
-    GdkColor   color;
     GList *line;
-    glyph g;
-    int i, j;
+    tomoe_glyph g;
+    guint i, j;
+    GdkColor color;
+    color.red   = 0x8000;
+    color.green = 0x0000;
+    color.blue  = 0x0000;
 
     g_return_if_fail (TOMOE_IS_CANVAS (canvas));
 
+    _init_gc (canvas);
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
     g.stroke_num = g_list_length (priv->stroke_list);
-    g.strokes    = g_new0 (stroke, g.stroke_num);
+    g.strokes    = g_new0 (tomoe_stroke, g.stroke_num);
 
     for (i = 0, line = priv->stroke_list;
          line;
@@ -486,11 +523,11 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
         px = py = -1;
 
         g.strokes[i].point_num = g_list_length (dots);
-        g.strokes[i].points = g_new0 (point, g.strokes[i].point_num);
+        g.strokes[i].points = g_new0 (tomoe_point, g.strokes[i].point_num);
 
         for (j = 0, dot = dots; dot; j++, dot = dot->next) {
-            x = ((point *)(dot->data))->x;
-            y = ((point *)(dot->data))->y;
+            x = ((tomoe_point *)(dot->data))->x;
+            y = ((tomoe_point *)(dot->data))->y;
 
             g.strokes[i].points[j].x = x
                 * ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / priv->size);
@@ -498,18 +535,7 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
                 * ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / priv->size);
 
             if (px != -1) {
-                if (!priv->adjust_line_gc) {
-                    /* FIXME: customize color */
-                    color.red   = 0x8000;
-                    color.green = 0x0000;
-                    color.blue  = 0x0000;
-                    priv->adjust_line_gc = gdk_gc_new (widget->window);
-                    tomoe_canvas_set_adjust_line_color (canvas, &color);
-                    gdk_gc_set_line_attributes (priv->adjust_line_gc, 1,
-                                                GDK_LINE_SOLID,
-                                                GDK_CAP_BUTT,
-                                                GDK_JOIN_BEVEL);
-                }
+
                 gdk_draw_line(priv->pixmap,
                               priv->adjust_line_gc,
                               px, py, x, y);
@@ -522,11 +548,12 @@ tomoe_canvas_real_find (TomoeCanvas *canvas)
     }
 
     if (priv->candidates) {
-        tomoe_free_matched (priv->candidates, priv->candidates_len);
+        tomoe_array_free (priv->candidates);
         priv->candidates     = NULL;
         priv->candidates_len = 0;
     }
-    priv->candidates_len = tomoe_get_matched (&g, &priv->candidates);
+    priv->candidates = tomoe_db_searchByStrokes (priv->database, &g);
+    priv->candidates_len = tomoe_array_size (priv->candidates);
     for (i = 0; i < g.stroke_num; i++)
         g_free (g.strokes[i].points);
     g_free (g.strokes);
@@ -546,16 +573,86 @@ tomoe_canvas_real_clear (TomoeCanvas *canvas)
 
     g_return_if_fail (TOMOE_IS_CANVAS (canvas));
 
+    _init_gc (canvas);
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
     tomoe_canvas_free_stroke_list (canvas);
     tomoe_canvas_draw_background (canvas, TRUE);
 
     if (priv->candidates) {
-        tomoe_free_matched (priv->candidates, priv->candidates_len);
+        tomoe_array_free (priv->candidates);
         priv->candidates     = NULL;
         priv->candidates_len = 0;
     }
+
+    tomoe_canvas_refresh (canvas);
+}
+
+tomoe_glyph *
+tomoe_canvas_get_glyph (TomoeCanvas *canvas)
+{
+    TomoeCanvasPriv *priv;
+    GList *line;
+    tomoe_glyph *g = (tomoe_glyph*)calloc(1, sizeof (tomoe_glyph));
+    int i, j;
+
+    g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), NULL);
+
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+
+    g->stroke_num = g_list_length (priv->stroke_list);
+    g->strokes    = g_new0 (tomoe_stroke, g->stroke_num);
+
+    for (i = 0, line = priv->stroke_list;
+         line;
+         i++, line = line->next)
+    {
+        GList *dots, *dot;
+        gint x, y, px, py;
+
+        dots = g_list_prepend (
+            get_vertex((GList*)line->data,
+                       (GList*)g_list_last((GList*)line->data)),
+            ((GList*)(line->data))->data);
+        px = py = -1;
+
+        g->strokes[i].point_num = g_list_length (dots);
+        g->strokes[i].points = g_new0 (tomoe_point, g->strokes[i].point_num);
+
+        for (j = 0, dot = dots; dot; j++, dot = dot->next) {
+            x = ((tomoe_point *)(dot->data))->x;
+            y = ((tomoe_point *)(dot->data))->y;
+
+            g->strokes[i].points[j].x = x
+                * ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / priv->size);
+            g->strokes[i].points[j].y = y
+                * ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / priv->size);
+            px = x;
+            py = y;
+        }
+
+        g_list_free (dots);
+    }
+
+    return g;
+}
+
+void 
+tomoe_canvas_set_preview_glyph (TomoeCanvas *canvas, tomoe_glyph *glyph)
+{
+    TomoeCanvasPriv *priv;
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    priv->preview = glyph;
+
+    if (GTK_WIDGET (canvas)->window)
+        tomoe_canvas_refresh (canvas);
+}
+
+void
+tomoe_canvas_lock (TomoeCanvas *canvas, gboolean lock)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    priv->locked = lock;
 }
 
 typedef struct _CharSize CharSize;
@@ -574,8 +671,8 @@ get_char_size (TomoeCanvas *canvas, CharSize *char_size)
     GList *node0, *node;
 
     GList *base_stroke = ((GList*)(priv->stroke_list))->data;
-    gint base_x = ((point*)(base_stroke->data))->x;
-    gint base_y = ((point*)(base_stroke->data))->y;
+    gint base_x = ((tomoe_point*)(base_stroke->data))->x;
+    gint base_y = ((tomoe_point*)(base_stroke->data))->y;
 
     char_size->x0 = base_x;
     char_size->y0 = base_y;
@@ -586,8 +683,8 @@ get_char_size (TomoeCanvas *canvas, CharSize *char_size)
         GList *stroke = (GList*) node0->data;
 
         for (node = stroke; node; node = g_list_next (node)) {
-            gint x = ((point*)(node->data))->x;
-            gint y = ((point*)(node->data))->y;
+            gint x = ((tomoe_point*)(node->data))->x;
+            gint y = ((tomoe_point*)(node->data))->y;
 
             char_size->x0 = MIN (char_size->x0, x);
             char_size->y0 = MIN (char_size->y0, y);
@@ -605,11 +702,11 @@ tomoe_canvas_resize (TomoeCanvas *canvas, gdouble x_rate, gdouble y_rate)
 
     for (node0 = priv->stroke_list; node0; node0 = g_list_next (node0)) {
         GList *stroke = (GList*) node0->data;
-
         GList *node1;
+
         for (node1 = stroke; node1; node1 = g_list_next (node1)) {
-            ((point*)(node1->data))->x *= x_rate;
-            ((point*)(node1->data))->y *= y_rate;
+            ((tomoe_point*)(node1->data))->x *= x_rate;
+            ((tomoe_point*)(node1->data))->y *= y_rate;
         }
     }
 }
@@ -622,11 +719,11 @@ tomoe_canvas_position (TomoeCanvas *canvas, gint dx, gint dy)
 
     for (node0 = priv->stroke_list; node0; node0 = g_list_next (node0)) {
         GList *stroke = (GList*) node0->data;
-
         GList *node1;
+
         for (node1 = stroke; node1; node1 = g_list_next (node1)) {
-            ((point*)(node1->data))->x += dx;
-            ((point*)(node1->data))->y += dy;
+            ((tomoe_point*)(node1->data))->x += dx;
+            ((tomoe_point*)(node1->data))->y += dy;
         }
     }
 }
@@ -676,26 +773,23 @@ tomoe_canvas_get_number_of_candidates (TomoeCanvas *canvas)
     return priv->candidates_len;
 }
 
-gchar *
+tomoe_char *
 tomoe_canvas_get_nth_candidate (TomoeCanvas *canvas, guint nth)
 {
     TomoeCanvasPriv *priv;
-    gchar *utf8_str;
 
     g_return_val_if_fail (TOMOE_IS_CANVAS (canvas), NULL);
 
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
 
-    if (nth < priv->candidates_len) {
-        gsize bytes_read, bytes_written;
-        utf8_str = g_convert(priv->candidates[nth]->letter, -1,
-                             "UTF-8", "EUC-JP",
-                             &bytes_read, &bytes_written,
-                             NULL);
-        return utf8_str;
-    } else {
-        return g_strdup ("");
+    if (nth < priv->candidates_len)
+    {
+        tomoe_candidate* cand;
+        cand = tomoe_array_get (priv->candidates, nth);
+        return cand->character;
     }
+    else
+        return NULL;
 }
 
 void
@@ -708,6 +802,8 @@ tomoe_canvas_refresh (TomoeCanvas *canvas)
 
     g_return_if_fail (TOMOE_IS_CANVAS (canvas));
 
+    _init_gc (canvas);
+
     priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     widget = GTK_WIDGET (canvas);
 
@@ -720,6 +816,14 @@ tomoe_canvas_refresh (TomoeCanvas *canvas)
 
     tomoe_canvas_draw_axis (canvas);
 
+    if (priv->preview) {
+        guint i;
+        index = 1;
+        for (i = 0; i < priv->preview->stroke_num; i++)
+            draw_preview_stroke (canvas, &priv->preview->strokes[i], &index);
+    }
+
+    index = 1;
     for (node = priv->stroke_list; node; node = g_list_next (node)) {
         draw_stroke (node->data, canvas, &index);
     }
@@ -798,10 +902,11 @@ tomoe_canvas_set_size (TomoeCanvas *canvas, guint size)
     priv->size = size;
 }
 
-void
+static void
 tomoe_canvas_gc_set_foreground (GdkGC *gc, GdkColor *color)
 {
     GdkColor default_color;
+
     default_color.red   = 0x0000;
     default_color.green = 0x0000;
     default_color.blue  = 0x0000;
@@ -887,55 +992,55 @@ tomoe_canvas_get_auto_find_time (TomoeCanvas *canvas)
     return priv->auto_find_time;
 }
 
+void
+tomoe_canvas_set_database (TomoeCanvas *canvas, tomoe_db *database)
+{
+    TomoeCanvasPriv *priv;
+    g_return_if_fail (TOMOE_IS_CANVAS (canvas));
+    priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    priv->database = database;
+}
+
 static void
 tomoe_canvas_append_point (TomoeCanvas *canvas, gint x, gint y)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     GList *last;
-    point *p1, *p2 = g_new0 (point, 1);
+    tomoe_point *p1, *p2 = g_new0 (tomoe_point, 1);
+    GdkColor color;
+    color.red   = 0x0000;
+    color.green = 0x0000;
+    color.blue  = 0x0000;
 
     p2->x = x;
     p2->y = y;
 
     last = g_list_last (priv->stroke);
     priv->stroke = g_list_append (priv->stroke, p2);
-    p1 = last ? (point*) last->data : p2;
+    p1 = last ? (tomoe_point*) last->data : p2;
 
-    tomoe_canvas_draw_line (canvas, p1, p2, TRUE);
+    _init_gc (canvas);
+    tomoe_canvas_draw_line (canvas, p1, p2, TRUE, &color);
 }
 
 static void
-tomoe_canvas_draw_line (TomoeCanvas *canvas, point *p1, point *p2,
-                        gboolean draw)
+tomoe_canvas_draw_line (TomoeCanvas *canvas,
+                        tomoe_point *p1, tomoe_point *p2,
+                        gboolean draw, GdkColor *color)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     GtkWidget *widget = GTK_WIDGET (canvas);
-    GdkColor   color;
-    GdkRectangle update_rect;
+    gint x, y, width, height;
 
-    /* FIXME! move to realize method */
-    if (!priv->handwrite_line_gc) {
-        /* FIXME: customize color */
-        color.red   = 0x0000;
-        color.green = 0x0000;
-        color.blue  = 0x0000;
-        priv->handwrite_line_gc = gdk_gc_new (widget->window);
-        tomoe_canvas_set_handwrite_line_color (canvas, &color);
-        gdk_gc_set_line_attributes (priv->handwrite_line_gc, 4,
-                                    GDK_LINE_SOLID,
-                                    GDK_CAP_ROUND,
-                                    GDK_JOIN_ROUND);
-    }
-
-    update_rect.x = MIN (p1->x, p2->x) - 2;
-    update_rect.y = MIN (p1->y, p2->y) - 2;
-    update_rect.width  = abs (p1->x - p2->x) + (2 * 2);
-    update_rect.height = abs (p1->y - p2->y) + (2 * 2);
+    x = MIN (p1->x, p2->x) - 2;
+    y = MIN (p1->y, p2->y) - 2;
+    width  = abs (p1->x - p2->x) + (2 * 2);
+    height = abs (p1->y - p2->y) + (2 * 2);
 
     gdk_draw_line (priv->pixmap, priv->handwrite_line_gc,
                    p1->x, p1->y, p2->x, p2->y);
     if (draw) {
-        gtk_widget_draw (widget, &update_rect);
+        gtk_widget_queue_draw_area (widget, x, y, width, height);
     }
 }
 
@@ -997,6 +1102,51 @@ tomoe_canvas_draw_axis (TomoeCanvas *canvas)
                    priv->size, (priv->size / 2));
 }
 
+static void
+_init_gc (TomoeCanvas *canvas)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    GtkWidget *widget = GTK_WIDGET (canvas);
+
+    if (!priv->adjust_line_gc)
+    {
+        GdkColor color;
+        color.red   = 0x8000;
+        color.green = 0x0000;
+        color.blue  = 0x0000;
+        priv->adjust_line_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_adjust_line_color (canvas, &color);
+        gdk_gc_set_line_attributes (priv->adjust_line_gc, 1,
+                                    GDK_LINE_SOLID,
+                                    GDK_CAP_BUTT,
+                                    GDK_JOIN_BEVEL);
+    }
+
+    if (!priv->handwrite_line_gc)
+    {
+        GdkColor color;
+        color.red   = 0x0000;
+        color.green = 0x0000;
+        color.blue  = 0x0000;
+        priv->handwrite_line_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_handwrite_line_color (canvas, &color);
+        gdk_gc_set_line_attributes (priv->handwrite_line_gc, 4,
+                                    GDK_LINE_SOLID,
+                                    GDK_CAP_ROUND,
+                                    GDK_JOIN_ROUND);
+    }
+
+    if (!priv->annotate_gc)
+    {
+        GdkColor color;
+        color.red   = 0x8000;
+        color.green = 0x0000;
+        color.blue  = 0x0000;
+        priv->annotate_gc = gdk_gc_new (widget->window);
+        tomoe_canvas_set_annotate_color (canvas, &color);
+    }
+}
+
 static gint
 get_distance (GList *first_node, GList *last_node, GList **most_node)
 {
@@ -1011,9 +1161,9 @@ get_distance (GList *first_node, GList *last_node, GList **most_node)
     gint dist = 0;
     gint max  = 0;
     gint denom;
-    point *first = (point*) first_node->data;
-    point *last  = (point*) last_node->data;
-    point *p;
+    tomoe_point *first = (tomoe_point*) first_node->data;
+    tomoe_point *last  = (tomoe_point*) last_node->data;
+    tomoe_point *p;
 
     *most_node = NULL;
     if (first_node == last_node)
@@ -1027,7 +1177,7 @@ get_distance (GList *first_node, GList *last_node, GList **most_node)
 
     for (dot = first_node; dot != last_node; dot = dot->next)
     {
-        p = (point*) dot->data;
+        p = (tomoe_point*) dot->data;
         dist = abs((a * p->y) - (b * p->x) + c);
         if (dist > max)
         {
@@ -1070,24 +1220,59 @@ static void
 draw_stroke (GList *stroke, TomoeCanvas *canvas, guint *index)
 {
     GList *node;
-    
+    GdkColor color;
+    color.red   = 0x8000;
+    color.green = 0x0000;
+    color.blue  = 0x0000;
+
+    _init_gc (canvas);
+
     for (node = stroke; node; node = g_list_next (node)) {
         GList *next = g_list_next (node);
+        tomoe_point *p1, *p2;
+
         if (!next) break;
 
-        point *p1 = (point*) node->data, *p2 = (point*) next->data;
-        tomoe_canvas_draw_line (canvas, p1, p2, FALSE);
+        p1 = (tomoe_point*) node->data;
+        p2 = (tomoe_point*) next->data;
+
+        tomoe_canvas_draw_line (canvas, p1, p2, FALSE, &color);
     }
-    draw_annotate (stroke, canvas, *index);
+    draw_annotate (stroke, canvas, *index, &color);
     (*index)++;
 }
 
 static void
-draw_annotate (GList *stroke, TomoeCanvas *canvas, guint index)
+draw_preview_stroke (TomoeCanvas *canvas, tomoe_stroke* stroke, guint *index)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    guint i;
+    GdkColor   color;
+    color.red   = 0x0000;
+    color.green = 0x0000;
+    color.blue  = 0x8000;
+
+    _init_gc (canvas);
+
+    for (i = 0; i < stroke->point_num - 1; i++) {
+        tomoe_point p1 = stroke->points[i];
+        p1.x = (gdouble)p1.x / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / (gdouble)priv->size);
+        p1.y = (gdouble)p1.y / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / priv->size);
+        tomoe_point p2 = stroke->points[i + 1];
+        p2.x = (gdouble)p2.x / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / (gdouble)priv->size);
+        p2.y = (gdouble)p2.y / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / (gdouble)priv->size);
+
+        tomoe_canvas_draw_line (canvas, &p1, &p2, FALSE, &color);
+    }
+    /* draw_preview_annotate (canvas, stroke, *index, &color); */
+    (*index)++;
+}
+
+static void
+draw_annotate (GList *stroke, TomoeCanvas *canvas, guint index, GdkColor *color)
 {
     TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
     GtkWidget *widget = GTK_WIDGET (canvas);
-    GdkColor   color;
 
     gchar *buffer; 
     PangoLayout *layout;
@@ -1098,15 +1283,17 @@ draw_annotate (GList *stroke, TomoeCanvas *canvas, guint index)
     gdouble dl;
     gint sign;
 
-    x = ((point*)((g_list_first (stroke))->data))->x;
-    y = ((point*)((g_list_first (stroke))->data))->y;
+    gdouble factor = 1.0 / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / (gdouble)priv->size);
+
+    x = ((tomoe_point*)((g_list_first (stroke))->data))->x * factor;
+    y = ((tomoe_point*)((g_list_first (stroke))->data))->y * factor;
 
     if (g_list_length (stroke) == 1) {
         dx = x;
         dy = y;
     } else {
-        dx = ((point*)((g_list_last (stroke))->data))->x - x;
-        dy = ((point*)((g_list_last (stroke))->data))->y - y;
+        dx = ((tomoe_point*)((g_list_last (stroke))->data))->x * factor - x;
+        dy = ((tomoe_point*)((g_list_last (stroke))->data))->y * factor - y;
     }
     dl = sqrt (dx*dx + dy*dy);
     sign = (dy <= dx) ? 1 : -1;
@@ -1117,17 +1304,9 @@ draw_annotate (GList *stroke, TomoeCanvas *canvas, guint index)
 
     r = sqrt (width*width + height*height);
 
-    x += 0.5 + (0.5 * r * dx / dl) + (sign * 0.5 * r * dy / dl) - (width / 2);
-    y += 0.5 + (0.5 * r * dy / dl) - (sign * 0.5 * r * dx / dl) - (height / 2);
+    x += (0.5 + (0.5 * r * dx / dl) + (sign * 0.5 * r * dy / dl) - (width / 2));
+    y += (0.5 + (0.5 * r * dy / dl) - (sign * 0.5 * r * dx / dl) - (height / 2));
 
-    if (!priv->annotate_gc) {
-        /* FIXME: customize color */
-        color.red   = 0x8000;
-        color.green = 0x0000;
-        color.blue  = 0x0000;
-        priv->annotate_gc = gdk_gc_new (widget->window);
-        tomoe_canvas_set_annotate_color (canvas, &color);
-    }
 
     gdk_draw_layout (priv->pixmap,
                      priv->annotate_gc,
@@ -1136,3 +1315,61 @@ draw_annotate (GList *stroke, TomoeCanvas *canvas, guint index)
     g_free (buffer);
     g_object_unref (layout);
 }
+
+static void
+draw_preview_annotate (TomoeCanvas *canvas, tomoe_stroke *stroke, guint index, GdkColor *color)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (canvas);
+    GtkWidget *widget = GTK_WIDGET (canvas);
+
+    if (stroke->point_num <= 0) return;
+
+    gchar *buffer; 
+    PangoLayout *layout;
+    gint width, height;
+    gint x, y;
+    gdouble r;
+    gdouble dx,dy;
+    gdouble dl;
+    gint sign;
+    gdouble factor = 1.0 / ((gdouble)TOMOE_CANVAS_DEFAULT_SIZE / (gdouble)priv->size);
+
+    x = stroke->points[0].x * factor;
+    y = stroke->points[0].y * factor;
+
+    if (stroke->point_num == 1) {
+        dx = x;
+        dy = y;
+    } else {
+        dx = stroke->points[stroke->point_num - 1].x * factor - x;
+        dy = stroke->points[stroke->point_num - 1].y * factor - y;
+    }
+    dl = sqrt (dx*dx + dy*dy);
+    sign = (dy <= dx) ? 1 : -1;
+
+    buffer = g_strdup_printf ("%d", index);
+    layout = gtk_widget_create_pango_layout (widget, buffer);
+    pango_layout_get_pixel_size (layout, &width, &height);
+
+    r = sqrt (width*width + height*height);
+
+    x += (0.5 + (0.5 * r * dx / dl) + (sign * 0.5 * r * dy / dl) - (width / 2));
+    y += (0.5 + (0.5 * r * dy / dl) - (sign * 0.5 * r * dx / dl) - (height / 2));
+
+    gdk_draw_layout (priv->pixmap,
+                     priv->annotate_gc,
+                     x, y, layout);
+
+    g_free (buffer);
+    g_object_unref (layout);
+}
+
+static void
+on_size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
+{
+    TomoeCanvasPriv *priv = TOMOE_CANVAS_GET_PRIVATE (TOMOE_CANVAS (widget));
+    priv->size = allocation->width;
+
+    if (widget->window)
+        tomoe_canvas_refresh (TOMOE_CANVAS (widget));
+}
